@@ -43,7 +43,7 @@ from __future__ import with_statement
 import base64, hashlib, os, re, string, tempfile, subprocess, types, functools, StringIO
 import fabric, fabric.api, fabric.operations, fabric.context_managers
 
-VERSION         = "0.5.1"
+VERSION         = "0.5.2"
 RE_SPACES       = re.compile("[\s\t]+")
 MAC_EOL         = "\n"
 UNIX_EOL        = "\n"
@@ -55,7 +55,7 @@ OPTION_PACKAGE  = "CUISINE_OPTION_PACKAGE"
 OPTION_PYTHON_PACKAGE  = "CUISINE_OPTION_PYTHON_PACKAGE"
 
 AVAILABLE_OPTIONS = dict(
-	package=["apt", "yum", "zypper"],
+	package=["apt", "yum", "zypper", "pacman"],
 	python_package=["easy_install","pip"]
 )
 
@@ -136,7 +136,7 @@ def connect( host, user="root" ):
 # =============================================================================
 
 def select_package( selection=None ):
-	"""Selects the type of package subsystem to use (ex:apt, yum or zypper)."""
+	"""Selects the type of package subsystem to use (ex:apt, yum, zypper, or pacman)."""
 	supported = AVAILABLE_OPTIONS["package"]
 	if not (selection is None):
 		assert selection in supported, "Option must be one of: %s"  % (supported)
@@ -404,7 +404,7 @@ def file_attribs_get(location):
 	else:
 		return None
 
-def file_write(location, content, mode=None, owner=None, group=None, sudo=None, check=True):
+def file_write(location, content, mode=None, owner=None, group=None, sudo=None, check=True, scp=False):
 	"""Writes the given content to the file at the given remote
 	location, optionally setting mode/owner/group."""
 	# FIXME: Big files are never transferred properly!
@@ -419,19 +419,25 @@ def file_write(location, content, mode=None, owner=None, group=None, sudo=None,
 			with mode_sudo(use_sudo):
 				run('cp "%s" "%s"'%(local_path,location))
 		else:
-			# FIXME: Put is not working properly, I often get stuff like:
-			# Fatal error: sudo() encountered an error (return code 1) while executing 'mv "3dcf7213c3032c812769e7f355e657b2df06b687" "/etc/authbind/byport/80"'
-			#fabric.operations.put(local_path, location, use_sudo=use_sudo)
-			# Hides the output, which is especially important
-			with fabric.context_managers.settings(
-				fabric.api.hide('stdout'),
-				warn_only=True,
-				**{MODE_SUDO: use_sudo}
-			):
-				# See: http://unix.stackexchange.com/questions/22834/how-to-uncompress-zlib-data-in-unix
-				result = run("echo '%s' | openssl base64 -A -d -out \"%s\"" % (base64.b64encode(content), location))
-				if "openssl:Error" in result:
-					fabric.api.abort('cuisine.file_write("%s",...) failed because openssl does not support base64 command.' % (location))
+			if scp:
+				hostname = fabric.api.env.host_string if len(fabric.api.env.host_string.split(':')) == 1 else fabric.api.env.host_string.split(':')[0]
+				scp_cmd = 'scp "%s" "%s"@"%s":"%s"'%(local_path,fabric.api.env.user,hostname,location)
+				print('[localhost] ' +  scp_cmd)
+				run_local(scp_cmd)
+			else:
+				# FIXME: Put is not working properly, I often get stuff like:
+				# Fatal error: sudo() encountered an error (return code 1) while executing 'mv "3dcf7213c3032c812769e7f355e657b2df06b687" "/etc/authbind/byport/80"'
+				#fabric.operations.put(local_path, location, use_sudo=use_sudo)
+				# Hides the output, which is especially important
+				with fabric.context_managers.settings(
+					fabric.api.hide('stdout'),
+					warn_only=True,
+					**{MODE_SUDO: use_sudo}
+				):
+					# See: http://unix.stackexchange.com/questions/22834/how-to-uncompress-zlib-data-in-unix
+					result = run("echo '%s' | openssl base64 -A -d -out \"%s\"" % (base64.b64encode(content), location))
+					if "openssl:Error" in result:
+						fabric.api.abort('cuisine.file_write("%s",...) failed because openssl does not support base64 command.' % (location))
 	# Remove the local temp file
 	os.fsync(fd)
 	os.close(fd)
@@ -444,15 +450,15 @@ def file_write(location, content, mode=None, owner=None, group=None, sudo=None,
 	with mode_sudo(use_sudo):
 		file_attribs(location, mode=mode, owner=owner, group=group)
 
-def file_ensure(location, mode=None, owner=None, group=None):
+def file_ensure(location, mode=None, owner=None, group=None, scp=False):
 	"""Updates the mode/owner/group for the remote file at the given
 	location."""
 	if file_exists(location):
 		file_attribs(location,mode=mode,owner=owner,group=group)
 	else:
-		file_write(location,"",mode=mode,owner=owner,group=group)
+		file_write(location,"",mode=mode,owner=owner,group=group,scp=scp)
 
-def file_upload(remote, local, sudo=None):
+def file_upload(remote, local, sudo=None, scp=False):
 	"""Uploads the local file to the remote location only if the remote location does not
 	exists or the content are different."""
 	# FIXME: Big files are never transferred properly!
@@ -468,7 +474,13 @@ def file_upload(remote, local, sudo=None):
 			else:
 				run('cp "%s" "%s"'%(local,remote))
 		else:
-			fabric.operations.put(local, remote, use_sudo=use_sudo)
+			if scp:
+				hostname = fabric.api.env.host_string if len(fabric.api.env.host_string.split(':')) == 1 else fabric.api.env.host_string.split(':')[0]
+				scp_cmd = 'scp "%s" "%s"@"%s":"%s"'%(local,fabric.api.env.user,hostname,remote)
+				print('[localhost] ' +  scp_cmd)
+				run_local(scp_cmd)
+			else:
+				fabric.operations.put(local, remote, use_sudo=use_sudo)
 
 def file_update(location, updater=lambda x: x):
 	"""Updates the content of the given by passing the existing
@@ -637,7 +649,8 @@ def package_remove(package, autoclean=False):
 # -----------------------------------------------------------------------------
 
 def repository_ensure_apt(repository):
-	sudo("add-apt-repository --yes" + repository)
+	package_ensure_apt('python-software-properties')
+	sudo("add-apt-repository --yes " + repository)
 
 def package_update_apt(package=None):
 	if package == None:
@@ -765,6 +778,54 @@ def package_ensure_zypper(package, update=False):
 def package_clean_zypper():
 	sudo("zypper --non-interactive clean")
 
+
+# -----------------------------------------------------------------------------
+# PACMAN PACKAGE (Arch)
+# -----------------------------------------------------------------------------
+
+def repository_ensure_pacman(repository):
+	raise Exception("Not implemented for Pacman")
+
+def package_update_pacman(package=None):
+	if package == None:
+		sudo("pacman --noconfirm -Sy")
+	else:
+		if type(package) in (list, tuple):
+			package = " ".join(package)
+		sudo("pacman --noconfirm -S " + package)
+
+def package_upgrade_pacman():
+	sudo("pacman --noconfirm -Syu")
+
+def package_install_pacman(package, update=False):
+	if update:
+		sudo("pacman --noconfirm -Sy")
+	if type(package) in (list, tuple):
+		package = " ".join(package)
+	sudo("pacman --noconfirm -S %s" % (package))
+
+def package_ensure_pacman(package, update=False):
+	"""Ensure apt packages are installed"""
+	if not isinstance(package, basestring):
+		package = " ".join(package)
+	status = run("pacman -Q %s ; true" % package)
+	if ('was not found' in status):
+		package_install_pacman(package, update)
+		return False
+	else:
+		if update:
+			package_update_pacman(package)
+		return True
+
+def package_clean_pacman():
+	sudo("pacman --noconfirm -Sc")
+
+def package_remove_pacman(package, autoclean=False):
+	if autoclean:
+		sudo('pacman --noconfirm -Rs ' + package)
+	else:
+		sudo('pacman --noconfirm -R ' + package)
+
 # =============================================================================
 #
 # PYTHON PACKAGE OPERATIONS
